;----------------------------------------------------------------------------
; Sega Master System & Game Gear - VDP functions
; by Omar Cornut / Bock|Zoop (c) in February 2001
;----------------------------------------------------------------------------

; VDP_INIT() ----------------------------------------------------------------
; Initialize default VDP registers, clear VRAM, clear sprites
;----------------------------------------------------------------------------
; no parameters
;----------------------------------------------------------------------------
vdp_init:
                in      a, (VDP_STATUS)         ; Read VDP status once
                ld      hl, vdp_init_table      ; before using VDP
                ld      b, 11*2                 ;
                ld      c, VDP_ADDR             ; Then upload default
                otir                            ; registers.
; VDP_CLEAR() ---------------------------------------------------------------
; Clear VRAM, clear sprites
;----------------------------------------------------------------------------
; no parameters
;----------------------------------------------------------------------------
vdp_clear:                                      ;
                ld      de, $0000               ; Clear VRAM
                rst     18h                     ; Set VDP address to DE
                ld      bc, VRAM_SIZE           ;
vdp_clear_loop:                                 ;
                xor     a
                out     (VDP_DATA), a           ;
                dec     bc                      ;
                ld      a, b                    ;
                or      c                       ;
                jr      nz, vdp_clear_loop      ;
; VDP_DISABLE_SPRITES() -----------------------------------------------------
; Clear sprites (by setting the first sprite position to 208)
;----------------------------------------------------------------------------
; no parameters
;----------------------------------------------------------------------------
vdp_disable_sprites:
                ld      de, VRAM_SPR_MAP+1      ; Disable sprites
                rst     18h                     ;
                ld      a, VRAM_SPR_LAST        ;
                out     (VDP_DATA), a           ;
                ret
;----------------------------------------------------------------------------
vdp_init_table:
.db             $36, $80,  $80, $81,  $FF, $82,  $FF, $83,  $FF, $84
.db             $FF, $85,  $FB, $86,  $00, $87,  $00, $88,  $00, $89
.db             $FF, $8A
;----------------------------------------------------------------------------

; VDP_LOAD_DATA() -----------------------------------------------------------
; Load data from given source to video memory
;----------------------------------------------------------------------------
; bc = number of bytes
; hl = source in ROM/RAM
; de = destination in VRAM
;----------------------------------------------------------------------------
vdp_load_data:
                push    hl
                rst     18H                     ; Set VDP address to DE
vdp_load_data_loop:
                ld      a, (hl)
                inc     hl
                out     (VDP_DATA), a
                dec     bc
                ld      a, b
                or      c
                jr      nz, vdp_load_data_loop
                pop     hl
                ret
;----------------------------------------------------------------------------

; VDP_BG_PUTIMAGE() ---------------------------------------------------------
; Put image to background tile map
;----------------------------------------------------------------------------
;  b = image width (in tile)
;  c = image height (in tile)
;  d = starting tile number
;  e = attribute (bit 0 must not be set)
; hl = VRAM address
;----------------------------------------------------------------------------
vdp_bg_putimage:
                push    bc
                push    de
                push    hl

vdp_bg_putimage_y:
                rst     28H                     ; Set VDP address to HL
                push    bc
vdp_bg_putimage_x:
                ld      a, d
                out     (VDP_DATA), a
                push    ix
                pop     ix
                ld      a, e
                out     (VDP_DATA), a
                inc     d
                jr      nz, vdp_bg_putimage_attr_end
                set     0, e
vdp_bg_putimage_attr_end:
                djnz    vdp_bg_putimage_x
                ld      bc, 32*2
                add     hl, bc
                pop     bc
                dec     c
                jr      nz, vdp_bg_putimage_y

                pop     hl
                pop     de
                pop     bc
                ret
;----------------------------------------------------------------------------

; VDP_BG_PUTIMAGE_FLIPV() ---------------------------------------------------
; Put image to background tile map, vertically flipped
;----------------------------------------------------------------------------
;  b = image width (in tile)
;  c = image height (in tile)
;  d = starting tile number
;  e = attribute (bit 0 must not be set)
; hl = VRAM address
;----------------------------------------------------------------------------
vdp_bg_putimage_flipv:
                push    bc
                push    de
                push    hl

                push    bc
                ld      a, c
                dec     a
                ld      bc, 32*2
vdp_bg_putimage_flipv_init:
                add     hl, bc
                dec     a
                jr      nz, vdp_bg_putimage_flipv_init
                pop     bc

vdp_bg_putimage_flipv_y:
                rst     28H                     ; Set VDP address to HL
                push    bc
vdp_bg_putimage_flipv_x:
                ld      a, d
                out     (VDP_DATA), a
                push    ix
                pop     ix
                ld      a, e
                out     (VDP_DATA), a
                inc     d
                jr      nz, vdp_bg_putimage_flipv_attr_end
                set     0, e
vdp_bg_putimage_flipv_attr_end:
                djnz    vdp_bg_putimage_flipv_x
; Gimme a macro!!
.include "dechl64.inc"
                pop     bc
                dec     c
                jr      nz, vdp_bg_putimage_flipv_y

                pop     hl
                pop     de
                pop     bc
                ret
;----------------------------------------------------------------------------

; VDP_BG_PUTIMAGE_FLIPH() ---------------------------------------------------
; Put image to background tile map, horizontally flipped
;----------------------------------------------------------------------------
;  b = image width (in tile)
;  c = image height (in tile)
;  d = starting tile number
;  e = attribute (bit 0 must not be set)
; hl = VRAM address
;----------------------------------------------------------------------------
vdp_bg_putimage_fliph:
                push    bc
                push    de
                push    hl

                ld      a, b
                add     a, d
                ld      d, a

vdp_bg_putimage_fliph_y:
                rst     28H                     ; Set VDP address to HL
                push    bc
vdp_bg_putimage_fliph_x:
                dec     d
                ld      a, d
                out     (VDP_DATA), a
                push    ix
                pop     ix
                ld      a, e
                out     (VDP_DATA), a
;                jr      nz, vdp_bg_putimage_fliph_attr_end
;                set     0, e
;vdp_bg_putimage_fliph_attr_end:
                djnz    vdp_bg_putimage_fliph_x
                ld      bc, 32*2
                add     hl, bc
                pop     bc

                ld      a, b
                rla
                add     a, d
                ld      d, a

                dec     c
                jr      nz, vdp_bg_putimage_fliph_y

                pop     hl
                pop     de
                pop     bc
                ret
;----------------------------------------------------------------------------

; VDP_BG_PUTIMAGE_FLIPVH() --------------------------------------------------
; Put image to background tile map, vertically and horizontally flipped
;----------------------------------------------------------------------------
;  b = image width (in tile)
;  c = image height (in tile)
;  d = starting tile number
;  e = attribute (bit 0 must not be set)
; hl = VRAM address
;----------------------------------------------------------------------------
vdp_bg_putimage_flipvh:
                push    bc
                push    de
                push    hl

                push    bc
                ld      a, c
                dec     a
                ld      bc, 32*2
vdp_bg_putimage_flipvh_init:
                add     hl, bc
                dec     a
                jr      nz, vdp_bg_putimage_flipvh_init
                pop     bc

                ld      a, b
                add     a, d
                ld      d, a

vdp_bg_putimage_flipvh_y:
                rst     28H                     ; Set VDP address to HL
                push    bc
vdp_bg_putimage_flipvh_x:
                dec     d
                ld      a, d
                out     (VDP_DATA), a
                push    ix
                pop     ix
                ld      a, e
                out     (VDP_DATA), a
;                jr      nz, vdp_bg_putimage_flipvh_attr_end
;                set     0, e
;vdp_bg_putimage_flipvh_attr_end:
                djnz    vdp_bg_putimage_flipvh_x
.include "dechl64.inc"
                pop     bc

                ld      a, b
                rla
                add     a, d
                ld      d, a

                dec     c
                jr      nz, vdp_bg_putimage_flipvh_y

                pop     hl
                pop     de
                pop     bc
                ret
;----------------------------------------------------------------------------

; VDP_BG_PUTTILE() ----------------------------------------------------------
; Put a tile to background tile map, starting at *current* VRAM address
;----------------------------------------------------------------------------
;  b = number of times
;  d = tile number
;  e = attribute (bit 0 must not be set)
; hl = VRAM address
;----------------------------------------------------------------------------
vdp_bg_puttile:
                rst     28H                     ; Set VDP address to HL
                push    ix
                pop     ix
vdp_bg_puttile_x:
                ld      a, d
                out     (VDP_DATA), a
                push    ix
                pop     ix
                ld      a, e
                out     (VDP_DATA), a
                djnz    vdp_bg_puttile_x
                ret
;----------------------------------------------------------------------------

; VDP_FRAME() / VDP_FRAME_ONE() ---------------------------------------------
; Wait for one or more frame to pass
;----------------------------------------------------------------------------
;  b = number of frames to wait for
;----------------------------------------------------------------------------
vdp_frame_one:
        ld      b, 1
vdp_frame:
        xor     a
        ld      (VAR_frame_cnt), a
vdp_frame_loop:
        ld      a, (VAR_frame_cnt)
        and     $FF
        jr      z, vdp_frame_loop
        djnz    vdp_frame
        ret
;----------------------------------------------------------------------------

; VDP_SET_PAL() -------------------------------------------------------------
; Set palette
;----------------------------------------------------------------------------
;  a = starting color
;  b = number of colors
; hl = data source
;----------------------------------------------------------------------------
vdp_set_pal:
                out     (VDP_ADDR), a           ;
                ld      a, 11000000b            ;
                out     (VDP_ADDR), a           ;
vdp_set_pal_loop:                               ;
                ld      a, (hl)                 ;
                out     (VDP_DATA), a           ;
                inc     hl                      ;
                djnz    vdp_set_pal_loop        ;
                ret                             ;
;----------------------------------------------------------------------------

; VDP_FADE() ----------------------------------------------------------------
; Fade palette
;----------------------------------------------------------------------------
; hl = address of fade data
;  a = starting affected color
;  b = number of colors per step
;  c = number of steps
;  d = tempo between steps
;----------------------------------------------------------------------------
vdp_fade:
                push    af
                push    bc
                call    vdp_set_pal
                ld      b, d
                call    vdp_frame
                pop     bc
                pop     af
                dec     c
                jr      nz, vdp_fade
                ret
;----------------------------------------------------------------------------

; VDP_SPRITE_BLIT() ---------------------------------------------------------
; Copy sprite tables to videoram
;----------------------------------------------------------------------------
; no parameters
;----------------------------------------------------------------------------
vdp_sprite_blit:
                ld      hl, VRAM_SPR_MAP
                rst     28h
                push    ix
                pop     ix
                ld      b, 64
                ld      c, VDP_DATA
                ld      hl, VAR_sprites_y
                otir
                ld      hl, VRAM_SPR_MAP+128
                rst     28h
                push    ix
                pop     ix
                ld      b, 128
                ld      c, VDP_DATA
                ld      hl, VAR_sprites_xn
                otir
                ret
;----------------------------------------------------------------------------

